@@ -1,9 +1,11 @@
 import logging
 import os
 import re
-from pathlib import Path
-from typing import List, Union, Optional
 from contextlib import suppress
+from pathlib import Path
+from typing import List, Optional, Union
+
+import torch
 import torch.utils.cpp_extension as torch_cpp_ext
 from filelock import FileLock
 
@@ -54,8 +56,9 @@ def check_cuda_arch():
 
 def clear_cache_dir():
     if os.path.exists(FLASHINFER_JIT_DIR):
-        for file in os.listdir(FLASHINFER_JIT_DIR):
-            os.remove(os.path.join(FLASHINFER_JIT_DIR, file))
+        import shutil
+
+        shutil.rmtree(FLASHINFER_JIT_DIR)
 
 
 def remove_unwanted_pytorch_nvcc_flags():
@@ -117,7 +120,7 @@ def load_cuda_ops(
     ] + CUTLASS_INCLUDE_DIRS
     lock = FileLock(FLASHINFER_JIT_DIR / f"{name}.lock", thread_local=False)
     with lock:
-        module = torch_cpp_ext.load(
+        torch_cpp_ext.load(
             name,
             list(map(lambda _: str(_), sources)),
             extra_cflags=cflags,
@@ -127,6 +130,9 @@ def load_cuda_ops(
             build_directory=build_directory,
             verbose=verbose,
             with_cuda=True,
+            # We switched to torch.library, so will be loaded into torch.ops
+            # instead of into a separate module.
+            is_python_module=False,
         )
     logger.info(f"Finished loading JIT ops: {name}")
-    return module
+    return getattr(torch.ops, name)
