@@ -20,17 +20,31 @@ from typing import List, Literal, Optional, Tuple, Union, overload
 
 import torch
 
-from .jit import gen_batch_mla_module, get_batch_mla_uri
-from .utils import MaskMode, get_cuda_stream, register_custom_op, register_fake_op
+from .jit import gen_batch_mla_module
+from .utils import (
+    MaskMode,
+    _check_shape_dtype_device,
+    determine_mla_backend,
+    get_cuda_stream,
+    register_custom_op,
+    register_fake_op,
+)
 
 _batch_mla_modules = {}
+_batch_mla_sm90_modules = {}
 
 
-def get_batch_mla_module(*args):
-    global _batch_mla_modules
-    if args not in _batch_mla_modules:
-        _batch_mla_modules[args] = gen_batch_mla_module(*args)
-    return _batch_mla_modules[args]
+def get_batch_mla_module(backend):
+    def backend_module(*args):
+        global _batch_mla_modules, _batch_mla_sm90_modules
+        modules_dict = (
+            _batch_mla_modules if backend == "fa2" else _batch_mla_sm90_modules
+        )
+        if args not in modules_dict:
+            modules_dict[args] = gen_batch_mla_module(backend, *args)
+        return modules_dict[args]
+
+    return backend_module
 
 
 class BatchMLAPagedAttentionWrapper:
@@ -107,7 +121,7 @@ class BatchMLAPagedAttentionWrapper:
         kv_indptr: Optional[torch.Tensor] = None,
         kv_indices: Optional[torch.Tensor] = None,
         kv_len_arr: Optional[torch.Tensor] = None,
-        backend: str = "fa2",
+        backend: str = "auto",
     ) -> None:
         r"""Constructor for BatchMLAPagedAttentionWrapper.
 
@@ -137,7 +151,9 @@ class BatchMLAPagedAttentionWrapper:
             should be ``[batch_size]``.
             This argument is only effective when ``use_cuda_graph`` is ``True``.
         backend : str
-            The implementation backend, default is "fa2".
+            The implementation backend, could be ``auto``/``fa2`` or ``fa3``. Defaults to ``auto``.
+            If set to ``auto``, the function will automatically choose the backend based on the
+            device architecture and kernel availability.
         """
         self._float_workspace_buffer = float_workspace_buffer
         self.device = float_workspace_buffer.device
@@ -154,6 +170,10 @@ class BatchMLAPagedAttentionWrapper:
         self._kv_indptr_buf = kv_indptr
         self._kv_indices_buf = kv_indices
         self._kv_len_arr_buf = kv_len_arr
+        if backend == "auto":
+            self._backend = determine_mla_backend(self.device)
+        else:
+            self._backend = backend
 
     def plan(
         self,
@@ -201,7 +221,7 @@ class BatchMLAPagedAttentionWrapper:
         kv_data_type : torch.dtype
             The data type of the kv-cache tensor.
         """
-        self._cached_module = get_batch_mla_module(
+        self._cached_module = get_batch_mla_module(self._backend)(
             q_data_type,
             kv_data_type,
             q_data_type,
@@ -267,6 +287,8 @@ class BatchMLAPagedAttentionWrapper:
         q_pe: torch.Tensor,
         ckv_cache: torch.Tensor,
         kpe_cache: torch.Tensor,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: bool = False,
     ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         r"""Run the MLA attention computation.
@@ -283,6 +305,10 @@ class BatchMLAPagedAttentionWrapper:
         kpe_cache : torch.Tensor
             The rope part of the kv-cache tensor, shape: ``[num_pages, page_size, head_dim_kpe]``.
             ``head_dim_kpe`` is 64 in DeepSeek v2/v3 models.
+        out : Optional[torch.Tensor]
+            The output tensor, if not provided, will be allocated internally.
+        lse : Optional[torch.Tensor]
+            The log-sum-exp of attention logits, if not provided, will be allocated internally.
         return_lse : bool, optional
             Whether to return the log-sum-exp value, default is False.
         """
@@ -292,12 +318,22 @@ class BatchMLAPagedAttentionWrapper:
         causal = self._causal
         mask_mode = MaskMode.CAUSAL.value if causal else MaskMode.NON_CAUSAL.value
         with self.device as device:
-            o = torch.empty_like(q_nope)
-            maybe_lse = (
-                torch.empty(q_nope.shape[:2], dtype=torch.float32, device=device)
-                if return_lse
-                else None
-            )
+            if out is None:
+                out = torch.empty_like(q_nope)
+            else:
+                _check_shape_dtype_device(
+                    out, q_nope.shape, q_nope.dtype, q_nope.device, "out"
+                )
+
+            if return_lse:
+                if lse is None:
+                    lse = torch.empty(
+                        q_nope.shape[:2], dtype=torch.float32, device=device
+                    )
+                else:
+                    _check_shape_dtype_device(
+                        lse, q_nope.shape[:2], torch.float32, q_nope.device, "lse"
+                    )
             self._cached_module.run(
                 self._float_workspace_buffer,
                 self._int_workspace_buffer,
@@ -307,8 +343,8 @@ class BatchMLAPagedAttentionWrapper:
                 ckv_cache,
                 kpe_cache,
                 self._kv_indices_buf,
-                o,
-                maybe_lse,
+                out,
+                lse,
                 mask_mode,
                 num_heads,
                 page_size,
@@ -316,4 +352,4 @@ class BatchMLAPagedAttentionWrapper:
                 get_cuda_stream(device),
             )
 
-        return (o, maybe_lse) if return_lse else o
+        return (out, lse) if return_lse else out
