@@ -37,7 +37,7 @@ def get_page_module():
     global _page_module
     if _page_module is None:
         if has_prebuilt_ops:
-            from . import _kernels
+            _kernels = torch.ops.flashinfer_kernels
 
             _page_module = _kernels
         else:
@@ -180,9 +180,9 @@ def get_batch_indices_positions(
     Returns
     -------
     batch_indices: torch.Tensor
-        The batch indices of the each entry in the ragged tensor, shape: ``[nnz]``.
+        The batch indices of each entry in the ragged tensor, shape: ``[nnz]``.
     positions: torch.Tensor
-        The positions of the each entry in the ragged tensor, shape: ``[nnz]``.
+        The positions of each entry in the ragged tensor, shape: ``[nnz]``.
 
     Example
     -------
@@ -201,7 +201,7 @@ def get_batch_indices_positions(
     ----
     This function is similar to `CSR2COO <https://docs.nvidia.com/cuda/cusparse/#csr2coo>`_
     conversion in cuSPARSE library, with the difference that we are converting from a ragged
-    tensor (which don't require a column indices array) to a COO format.
+    tensor (which doesn't require a column indices array) to a COO format.
 
     See Also
     --------
@@ -349,7 +349,7 @@ def append_paged_kv_cache(
 
     Note
     ----
-    The function assumes that the space for appended k/v have already been allocated,
+    The function assumes that the space for appended k/v has already been allocated,
     which means :attr:`kv_indices`, :attr:`kv_indptr`, :attr:`kv_last_page_len` has
     incorporated appended k/v.
 
