@@ -40,6 +40,7 @@ from .utils import (
     _check_cached_qkv_data_type,
     _check_kv_layout,
     _check_pos_encoding_mode,
+    _check_shape_dtype_device,
     _get_cache_alibi_slopes_buf,
     _get_cache_buf,
     _unpack_paged_kv_cache,
@@ -70,11 +71,11 @@ def get_single_prefill_module(backend):
             uri = get_single_prefill_uri(backend, *args)
             if has_prebuilt_ops and uri in prebuilt_ops_uri:
                 if backend == "fa2":
-                    from . import _kernels
+                    _kernels = torch.ops.flashinfer_kernels
 
                     run_func = _kernels.single_prefill_with_kv_cache
                 else:
-                    from . import _kernels_sm90
+                    _kernels_sm90 = torch.ops.flashinfer_kernels_sm90
 
                     run_func = _kernels_sm90.single_prefill_with_kv_cache_sm90
             else:
@@ -177,13 +178,13 @@ def get_batch_prefill_module(backend):
             uri = get_batch_prefill_uri(backend, *args)
             if has_prebuilt_ops and uri in prebuilt_ops_uri:
                 if backend == "fa2":
-                    from . import _kernels
+                    _kernels = torch.ops.flashinfer_kernels
 
                     plan_func = _kernels.batch_prefill_with_kv_cache_plan
                     ragged_run_func = _kernels.batch_prefill_with_ragged_kv_cache_run
                     paged_run_func = _kernels.batch_prefill_with_paged_kv_cache_run
                 else:
-                    from . import _kernels_sm90
+                    _kernels_sm90 = torch.ops.flashinfer_kernels_sm90
 
                     plan_func = _kernels_sm90.batch_prefill_with_kv_cache_sm90_plan
                     ragged_run_func = (
@@ -990,7 +991,7 @@ class BatchPrefillWithPagedKVCacheWrapper:
     Note
     ----
     To accelerate computation, FlashInfer's batch prefill/append attention operators
-    creates some auxiliary data structures, these data structures can be reused across
+    create some auxiliary data structures, these data structures can be reused across
     multiple prefill/append attention calls (e.g. different Transformer layers). This
     wrapper class manages the lifecycle of these data structures.
     """
@@ -1502,6 +1503,8 @@ class BatchPrefillWithPagedKVCacheWrapper:
         *args,
         k_scale: Optional[float] = None,
         v_scale: Optional[float] = None,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: Literal[False] = False,
     ) -> torch.Tensor: ...
 
@@ -1513,6 +1516,8 @@ class BatchPrefillWithPagedKVCacheWrapper:
         *args,
         k_scale: Optional[float] = None,
         v_scale: Optional[float] = None,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: Literal[True] = True,
     ) -> Tuple[torch.Tensor, torch.Tensor]: ...
 
@@ -1523,6 +1528,8 @@ class BatchPrefillWithPagedKVCacheWrapper:
         *args,
         k_scale: Optional[float] = None,
         v_scale: Optional[float] = None,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: bool = False,
     ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         r"""Compute batch prefill/append attention between query and paged kv-cache.
@@ -1545,10 +1552,16 @@ class BatchPrefillWithPagedKVCacheWrapper:
               :attr:`kv_layout` is ``HND``. Where ``paged_kv_cache[:, 0]`` is the key-cache and
               ``paged_kv_cache[:, 1]`` is the value-cache.
 
+        *args
+            Additional arguments for custom kernels.
         k_scale : Optional[float]
             The calibration scale of key for fp8 input, if not provided, will be set to ``1.0``.
         v_scale : Optional[float]
             The calibration scale of value for fp8 input, if not provided, will be set to ``1.0``.
+        out : Optional[torch.Tensor]
+            The output tensor, if not provided, will be allocated internally.
+        lse : Optional[torch.Tensor]
+            The log-sum-exp of attention logits, if not provided, will be allocated internally.
         return_lse : bool
             Whether to return the logsumexp of attention output
 
@@ -1587,15 +1600,24 @@ class BatchPrefillWithPagedKVCacheWrapper:
             rope_scale = 1.0
         if rope_theta is None:
             rope_theta = 1e4
-        lse = None
         if return_lse:
-            lse = torch.empty(
-                (q.size(0), q.size(1)), dtype=torch.float32, device=q.device
-            )
+            if lse is None:
+                lse = torch.empty(
+                    (q.size(0), q.size(1)), dtype=torch.float32, device=q.device
+                )
+            else:
+                _check_shape_dtype_device(
+                    lse, (q.size(0), q.size(1)), torch.float32, q.device, "lse"
+                )
 
-        out = torch.empty(
-            q.shape[:-1] + v_cache.shape[-1:], dtype=q.dtype, device=q.device
-        )
+        if out is None:
+            out = torch.empty(
+                q.shape[:-1] + v_cache.shape[-1:], dtype=q.dtype, device=q.device
+            )
+        else:
+            _check_shape_dtype_device(
+                out, q.shape[:-1] + v_cache.shape[-1:], q.dtype, q.device, "out"
+            )
 
         if self._custom_mask_buf is not None:
             mask_mode = MaskMode.CUSTOM.value
@@ -1793,7 +1815,7 @@ class BatchPrefillWithRaggedKVCacheWrapper:
     Note
     ----
     To accelerate computation, FlashInfer's batch prefill/append attention operators
-    creates some auxiliary data structures, these data structures can be reused across
+    create some auxiliary data structures, these data structures can be reused across
     multiple prefill/append attention calls (e.g. different Transformer layers). This
     wrapper class manages the lifecycle of these data structures.
     """
@@ -2198,6 +2220,8 @@ class BatchPrefillWithRaggedKVCacheWrapper:
         k: torch.Tensor,
         v: torch.Tensor,
         *args,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: Literal[False] = False,
     ) -> torch.Tensor: ...
 
@@ -2208,6 +2232,8 @@ class BatchPrefillWithRaggedKVCacheWrapper:
         k: torch.Tensor,
         v: torch.Tensor,
         *args,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: Literal[True] = True,
     ) -> Tuple[torch.Tensor, torch.Tensor]: ...
 
@@ -2217,6 +2243,8 @@ class BatchPrefillWithRaggedKVCacheWrapper:
         k: torch.Tensor,
         v: torch.Tensor,
         *args,
+        out: Optional[torch.Tensor] = None,
+        lse: Optional[torch.Tensor] = None,
         return_lse: bool = False,
     ) -> Union[torch.Tensor, Tuple[torch.Tensor, torch.Tensor]]:
         r"""Compute batch prefill/append attention between query and kv-cache stored as
@@ -2230,6 +2258,12 @@ class BatchPrefillWithRaggedKVCacheWrapper:
             The key tensor, shape: ``[kv_indptr[-1], num_kv_heads, head_dim_qk]``
         v : torch.Tensor
             The value tensor, shape: ``[kv_indptr[-1], num_kv_heads, head_dim_vo]``
+        *args
+            Additional arguments for the custom kernel.
+        out : Optional[torch.Tensor]
+            The output tensor, if not provided, will be allocated internally.
+        lse : Optional[torch.Tensor]
+            The log-sum-exp of attention logits, if not provided, will be allocated internally.
         return_lse : bool
             Whether to return the logsumexp of attention output
 
@@ -2259,12 +2293,23 @@ class BatchPrefillWithRaggedKVCacheWrapper:
             rope_scale = 1.0
         if rope_theta is None:
             rope_theta = 1e4
-        lse = None
         if return_lse:
-            lse = torch.empty(
-                (q.size(0), q.size(1)), dtype=torch.float32, device=q.device
+            if lse is None:
+                lse = torch.empty(
+                    (q.size(0), q.size(1)), dtype=torch.float32, device=q.device
+                )
+            else:
+                _check_shape_dtype_device(
+                    lse, (q.size(0), q.size(1)), torch.float32, q.device, "lse"
+                )
+        if out is None:
+            out = torch.empty(
+                q.shape[:-1] + v.shape[-1:], dtype=q.dtype, device=q.device
+            )
+        else:
+            _check_shape_dtype_device(
+                out, q.shape[:-1] + v.shape[-1:], q.dtype, q.device, "out"
             )
-        out = torch.empty(q.shape[:-1] + v.shape[-1:], dtype=q.dtype, device=q.device)
 
         if is_float8(q):
             logging.warning(
